@@ -23,8 +23,9 @@ from capa.features.extractors.base_extractor import BBHandle, InsnHandle, Functi
 
 class IdaFeatureExtractor(FeatureExtractor):
     def __init__(self):
-        super(IdaFeatureExtractor, self).__init__()
+        super().__init__()
         self.global_features: List[Tuple[Feature, Address]] = []
+        self.global_features.extend(capa.features.extractors.ida.file.extract_file_format())
         self.global_features.extend(capa.features.extractors.ida.global_.extract_os())
         self.global_features.extend(capa.features.extractors.ida.global_.extract_arch())
 
