@@ -6,7 +6,8 @@
 #  is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and limitations under the License.
 import datetime
-from typing import Any, Dict, Tuple, Union, Optional
+import collections
+from typing import Any, Dict, List, Tuple, Union, Optional
 
 from pydantic import Field, BaseModel
 
@@ -15,6 +16,7 @@ import capa.engine
 import capa.features.common
 import capa.features.freeze as frz
 import capa.features.address
+import capa.features.freeze.features as frzf
 from capa.rules import RuleSet
 from capa.engine import MatchResults
 from capa.helpers import assert_never
@@ -23,6 +25,7 @@ from capa.helpers import assert_never
 class FrozenModel(BaseModel):
     class Config:
         frozen = True
+        extra = "forbid"
 
 
 class Sample(FrozenModel):
@@ -75,7 +78,7 @@ class Analysis(FrozenModel):
 class Metadata(FrozenModel):
     timestamp: datetime.datetime
     version: str
-    argv: Tuple[str, ...]
+    argv: Optional[Tuple[str, ...]]
     sample: Sample
     analysis: Analysis
 
@@ -99,64 +102,91 @@ class Metadata(FrozenModel):
                 rules=meta["analysis"]["rules"],
                 base_address=frz.Address.from_capa(meta["analysis"]["base_address"]),
                 layout=Layout(
-                    functions=[
+                    functions=tuple(
                         FunctionLayout(
                             address=frz.Address.from_capa(address),
-                            matched_basic_blocks=[
+                            matched_basic_blocks=tuple(
                                 BasicBlockLayout(address=frz.Address.from_capa(bb)) for bb in f["matched_basic_blocks"]
-                            ],
+                            ),
                         )
                         for address, f in meta["analysis"]["layout"]["functions"].items()
-                    ]
+                    )
                 ),
                 feature_counts=FeatureCounts(
                     file=meta["analysis"]["feature_counts"]["file"],
-                    functions=[
+                    functions=tuple(
                         FunctionFeatureCount(address=frz.Address.from_capa(address), count=count)
                         for address, count in meta["analysis"]["feature_counts"]["functions"].items()
-                    ],
+                    ),
                 ),
-                library_functions=[
+                library_functions=tuple(
                     LibraryFunction(address=frz.Address.from_capa(address), name=name)
                     for address, name in meta["analysis"]["library_functions"].items()
-                ],
+                ),
             ),
         )
 
+    def to_capa(self) -> Dict[str, Any]:
+        capa_meta = {
+            "timestamp": self.timestamp.isoformat(),
+            "version": self.version,
+            "sample": {
+                "md5": self.sample.md5,
+                "sha1": self.sample.sha1,
+                "sha256": self.sample.sha256,
+                "path": self.sample.path,
+            },
+            "analysis": {
+                "format": self.analysis.format,
+                "arch": self.analysis.arch,
+                "os": self.analysis.os,
+                "extractor": self.analysis.extractor,
+                "rules": self.analysis.rules,
+                "base_address": self.analysis.base_address.to_capa(),
+                "layout": {
+                    "functions": {
+                        f.address.to_capa(): {
+                            "matched_basic_blocks": [bb.address.to_capa() for bb in f.matched_basic_blocks]
+                        }
+                        for f in self.analysis.layout.functions
+                    }
+                },
+                "feature_counts": {
+                    "file": self.analysis.feature_counts.file,
+                    "functions": {fc.address.to_capa(): fc.count for fc in self.analysis.feature_counts.functions},
+                },
+                "library_functions": {lf.address.to_capa(): lf.name for lf in self.analysis.library_functions},
+            },
+        }
+
+        return capa_meta
+
+
+class CompoundStatementType:
+    AND = "and"
+    OR = "or"
+    NOT = "not"
+    OPTIONAL = "optional"
+
 
 class StatementModel(FrozenModel):
     ...
 
 
-class AndStatement(StatementModel):
-    type = "and"
-    description: Optional[str]
-
-
-class OrStatement(StatementModel):
-    type = "or"
-    description: Optional[str]
-
-
-class NotStatement(StatementModel):
-    type = "not"
-    description: Optional[str]
+class CompoundStatement(StatementModel):
+    type: str
+    description: Optional[str] = None
 
 
 class SomeStatement(StatementModel):
     type = "some"
-    description: Optional[str]
+    description: Optional[str] = None
     count: int
 
 
-class OptionalStatement(StatementModel):
-    type = "optional"
-    description: Optional[str]
-
-
 class RangeStatement(StatementModel):
     type = "range"
-    description: Optional[str]
+    description: Optional[str] = None
     min: int
     max: int
     child: frz.Feature
@@ -164,18 +194,16 @@ class RangeStatement(StatementModel):
 
 class SubscopeStatement(StatementModel):
     type = "subscope"
-    description: Optional[str]
-    scope = capa.rules.Scope
+    description: Optional[str] = None
+    scope: capa.rules.Scope
 
 
 Statement = Union[
-    OptionalStatement,
-    AndStatement,
-    OrStatement,
-    NotStatement,
-    SomeStatement,
+    # Note! order matters, see #1161
     RangeStatement,
+    SomeStatement,
     SubscopeStatement,
+    CompoundStatement,
 ]
 
 
@@ -185,18 +213,12 @@ class StatementNode(FrozenModel):
 
 
 def statement_from_capa(node: capa.engine.Statement) -> Statement:
-    if isinstance(node, capa.engine.And):
-        return AndStatement(description=node.description)
-
-    elif isinstance(node, capa.engine.Or):
-        return OrStatement(description=node.description)
-
-    elif isinstance(node, capa.engine.Not):
-        return NotStatement(description=node.description)
+    if isinstance(node, (capa.engine.And, capa.engine.Or, capa.engine.Not)):
+        return CompoundStatement(type=node.__class__.__name__.lower(), description=node.description)
 
     elif isinstance(node, capa.engine.Some):
         if node.count == 0:
-            return OptionalStatement(description=node.description)
+            return CompoundStatement(type=CompoundStatementType.OPTIONAL, description=node.description)
 
         else:
             return SomeStatement(
@@ -241,7 +263,55 @@ def node_from_capa(node: Union[capa.engine.Statement, capa.engine.Feature]) -> N
         assert_never(node)
 
 
-class Match(BaseModel):
+def node_to_capa(
+    node: Node, children: List[Union[capa.engine.Statement, capa.engine.Feature]]
+) -> Union[capa.engine.Statement, capa.engine.Feature]:
+    if isinstance(node, StatementNode):
+        if isinstance(node.statement, CompoundStatement):
+            if node.statement.type == CompoundStatementType.AND:
+                return capa.engine.And(description=node.statement.description, children=children)
+
+            elif node.statement.type == CompoundStatementType.OR:
+                return capa.engine.Or(description=node.statement.description, children=children)
+
+            elif node.statement.type == CompoundStatementType.NOT:
+                return capa.engine.Not(description=node.statement.description, child=children[0])
+
+            elif node.statement.type == CompoundStatementType.OPTIONAL:
+                return capa.engine.Some(description=node.statement.description, count=0, children=children)
+
+            else:
+                assert_never(node.statement.type)
+
+        elif isinstance(node.statement, SomeStatement):
+            return capa.engine.Some(
+                description=node.statement.description, count=node.statement.count, children=children
+            )
+
+        elif isinstance(node.statement, RangeStatement):
+            return capa.engine.Range(
+                description=node.statement.description,
+                min=node.statement.min,
+                max=node.statement.max,
+                child=node.statement.child.to_capa(),
+            )
+
+        elif isinstance(node.statement, SubscopeStatement):
+            return capa.engine.Subscope(
+                description=node.statement.description, scope=node.statement.scope, child=children[0]
+            )
+
+        else:
+            assert_never(node.statement)
+
+    elif isinstance(node, FeatureNode):
+        return node.feature.to_capa()
+
+    else:
+        assert_never(node)
+
+
+class Match(FrozenModel):
     """
     args:
       success: did the node match?
@@ -293,7 +363,7 @@ class Match(BaseModel):
         # finally, splice that logic into this tree.
         if (
             isinstance(node, FeatureNode)
-            and isinstance(node.feature, frz.features.MatchFeature)
+            and isinstance(node.feature, frzf.MatchFeature)
             # only add subtree on success,
             # because there won't be results for the other rule on failure.
             and success
@@ -368,21 +438,54 @@ class Match(BaseModel):
         return cls(
             success=success,
             node=node,
+            children=tuple(children),
+            locations=tuple(locations),
+            captures={capture: tuple(captures[capture]) for capture in captures},
+        )
+
+    def to_capa(self, rules_by_name: Dict[str, capa.rules.Rule]) -> capa.engine.Result:
+        children = [child.to_capa(rules_by_name) for child in self.children]
+        statement = node_to_capa(self.node, [child.statement for child in children])
+
+        if isinstance(self.node, FeatureNode):
+            feature = self.node.feature
+
+            if isinstance(feature, (frzf.SubstringFeature, frzf.RegexFeature)):
+                matches = {capture: {loc.to_capa() for loc in locs} for capture, locs in self.captures.items()}
+
+                if isinstance(feature, frzf.SubstringFeature):
+                    assert isinstance(statement, capa.features.common.Substring)
+                    statement = capa.features.common._MatchedSubstring(statement, matches)
+                elif isinstance(feature, frzf.RegexFeature):
+                    assert isinstance(statement, capa.features.common.Regex)
+                    statement = capa.features.common._MatchedRegex(statement, matches)
+                else:
+                    assert_never(feature)
+
+        # apparently we don't have to fixup match and subscope entries here.
+        # at least, default, verbose, and vverbose renderers seem to work well without any special handling here.
+        #
+        # children contains a single tree of results, corresponding to the logic of the matched rule.
+        # self.node.feature.match contains the name of the rule that was matched.
+        # so its all available to reconstruct, if necessary.
+
+        return capa.features.common.Result(
+            success=self.success,
+            statement=statement,
+            locations={loc.to_capa() for loc in self.locations},
             children=children,
-            locations=locations,
-            captures=captures,
         )
 
 
 def parse_parts_id(s: str):
-    id = ""
+    id_ = ""
     parts = s.split("::")
     if len(parts) > 0:
         last = parts.pop()
-        last, _, id = last.rpartition(" ")
-        id = id.lstrip("[").rstrip("]")
+        last, _, id_ = last.rpartition(" ")
+        id_ = id_.lstrip("[").rstrip("]")
         parts.append(last)
-    return parts, id
+    return tuple(parts), id_
 
 
 class AttackSpec(FrozenModel):
@@ -408,7 +511,7 @@ class AttackSpec(FrozenModel):
         tactic = ""
         technique = ""
         subtechnique = ""
-        parts, id = parse_parts_id(s)
+        parts, id_ = parse_parts_id(s)
         if len(parts) > 0:
             tactic = parts[0]
         if len(parts) > 1:
@@ -421,7 +524,7 @@ class AttackSpec(FrozenModel):
             tactic=tactic,
             technique=technique,
             subtechnique=subtechnique,
-            id=id,
+            id=id_,
         )
 
 
@@ -448,7 +551,7 @@ class MBCSpec(FrozenModel):
         objective = ""
         behavior = ""
         method = ""
-        parts, id = parse_parts_id(s)
+        parts, id_ = parse_parts_id(s)
         if len(parts) > 0:
             objective = parts[0]
         if len(parts) > 1:
@@ -461,7 +564,7 @@ class MBCSpec(FrozenModel):
             objective=objective,
             behavior=behavior,
             method=method,
-            id=id,
+            id=id_,
         )
 
 
@@ -499,28 +602,30 @@ class RuleMetadata(FrozenModel):
             namespace=rule.meta.get("namespace"),
             authors=rule.meta.get("authors"),
             scope=capa.rules.Scope(rule.meta.get("scope")),
-            attack=list(map(AttackSpec.from_str, rule.meta.get("att&ck", []))),
-            mbc=list(map(MBCSpec.from_str, rule.meta.get("mbc", []))),
+            attack=tuple(map(AttackSpec.from_str, rule.meta.get("att&ck", []))),
+            mbc=tuple(map(MBCSpec.from_str, rule.meta.get("mbc", []))),
             references=rule.meta.get("references", []),
             examples=rule.meta.get("examples", []),
             description=rule.meta.get("description", ""),
             lib=rule.meta.get("lib", False),
-            capa_subscope=rule.meta.get("capa/subscope", False),
+            is_subscope_rule=rule.meta.get("capa/subscope", False),
             maec=MaecMetadata(
                 analysis_conclusion=rule.meta.get("maec/analysis-conclusion"),
                 analysis_conclusion_ov=rule.meta.get("maec/analysis-conclusion-ov"),
                 malware_family=rule.meta.get("maec/malware-family"),
                 malware_category=rule.meta.get("maec/malware-category"),
                 malware_category_ov=rule.meta.get("maec/malware-category-ov"),
-            ),
-        )
+            ),  # type: ignore
+            # Mypy is unable to recognise arguments due to alias
+        )  # type: ignore
+        # Mypy is unable to recognise arguments due to alias
 
     class Config:
         frozen = True
         allow_population_by_field_name = True
 
 
-class RuleMatches(BaseModel):
+class RuleMatches(FrozenModel):
     """
     args:
         meta: the metadata from the rule
@@ -532,7 +637,7 @@ class RuleMatches(BaseModel):
     matches: Tuple[Tuple[frz.Address, Match], ...]
 
 
-class ResultDocument(BaseModel):
+class ResultDocument(FrozenModel):
     meta: Metadata
     rules: Dict[str, RuleMatches]
 
@@ -548,10 +653,29 @@ class ResultDocument(BaseModel):
             rule_matches[rule_name] = RuleMatches(
                 meta=RuleMetadata.from_capa(rule),
                 source=rule.definition,
-                matches=[
+                matches=tuple(
                     (frz.Address.from_capa(addr), Match.from_capa(rules, capabilities, match))
                     for addr, match in matches
-                ],
+                ),
             )
 
         return ResultDocument(meta=Metadata.from_capa(meta), rules=rule_matches)
+
+    def to_capa(self) -> Tuple[Dict, Dict]:
+        meta = self.meta.to_capa()
+        capabilities: Dict[
+            str, List[Tuple[capa.features.address.Address, capa.features.common.Result]]
+        ] = collections.defaultdict(list)
+
+        # this doesn't quite work because we don't have the rule source for rules that aren't matched.
+        rules_by_name = {
+            rule_name: capa.rules.Rule.from_yaml(rule_match.source) for rule_name, rule_match in self.rules.items()
+        }
+
+        for rule_name, rule_match in self.rules.items():
+            for addr, match in rule_match.matches:
+                result: capa.engine.Result = match.to_capa(rules_by_name)
+
+                capabilities[rule_name].append((addr.to_capa(), result))
+
+        return meta, capabilities
